# This file adds functions to build the C library for bare metal (IE. none)
# Copyright 2008 Yann E. MORIN
# Licensed under the GPL v2. See COPYING in the root of this package

none_get()
{
    :
}

none_extract()
{
    :
}
