export install="/usr/bin/install -c"
export bash="/usr/bin/bash"
export bison="/usr/bin/bison"
export awk="/usr/bin/gawk"
export grep="/usr/bin/grep"
export gmake="/usr/bin/make"
export gnumake="/usr/bin/make"
export make="/usr/bin/make"
export python="python3.11"
export sed="/usr/bin/sed"
export libtool="/usr/bin/libtool"
export libtoolize="/usr/bin/libtoolize"
export objcopy="/usr/bin/objcopy"
export objdump="/usr/bin/objdump"
export readelf="/usr/bin/readelf"
export patch="/usr/bin/patch"
export md5sum="/usr/bin/md5sum"
export sha1sum="/usr/bin/sha1sum"
export sha256sum="/usr/bin/sha256sum"
export sha512sum="/usr/bin/sha512sum"
